"""
Tests the utility LibXC functions.
"""

import pylibxc


def test_xc_version():

    assert tuple(map(int, pylibxc.version.__version__.split('.'))) == pylibxc.util.xc_version()


def test_xc_string():

    assert pylibxc.version.__version__ == pylibxc.util.xc_version_string()


def test_xc_functional_get_number():

    assert 32 == pylibxc.util.xc_functional_get_number("XC_GGA_X_GAM")
    assert 32 == pylibxc.util.xc_functional_get_number("gga_x_gam")
    assert 32 == pylibxc.util.xc_functional_get_number("gga_x_gam".upper())

    assert -1 == pylibxc.util.xc_functional_get_number("nothing")


def test_xc_functional_get_name():
    assert "gga_x_gam" == pylibxc.util.xc_functional_get_name(32)
    assert None is pylibxc.util.xc_functional_get_name(50000)


def test_xc_family_from_id():
    assert (4, 3) == pylibxc.util.xc_family_from_id(72)
    assert (0, 0) == pylibxc.util.xc_family_from_id(500000)


def test_xc_number_of_functionals():
    assert pylibxc.util.xc_number_of_functionals() > 400


def test_xc_available_functional_numbers():
    func_nums = pylibxc.util.xc_available_functional_numbers()

    assert len(func_nums) == pylibxc.util.xc_number_of_functionals()

    # Spot check some functionals
    assert (1 in func_nums)
    assert (76 in func_nums)
    assert (540 in func_nums)

    # Make sure the range on all values is reasonable
    assert all(x > 0 for x in func_nums)
    assert all(x < 2000 or x > 100000 for x in func_nums)


def test_xc_available_functional_names():
    func_names = pylibxc.util.xc_available_functional_names()

    assert len(func_names) == pylibxc.util.xc_number_of_functionals()

    # Spot check some functionals
    assert ("lda_c_vwn" in func_names)
    assert ("gga_c_pbe" in func_names)
    assert ("gga_x_lbm" in func_names)

    # Make sure the range on all values is reasonable
    assert all((("x" in x) or ("c" in x) or ("k" in x)) for x in func_names)



"""
A port of the LibXC internal flags to Python. Header defines must be physically copied.
"""

XC_UNPOLARIZED = 1
XC_POLARIZED = 2

XC_NON_RELATIVISTIC = 0
XC_RELATIVISTIC = 1

XC_EXCHANGE = 0
XC_CORRELATION = 1
XC_EXCHANGE_CORRELATION = 2
XC_KINETIC = 3

XC_FAMILY_UNKNOWN = -1
XC_FAMILY_LDA = 1
XC_FAMILY_GGA = 2
XC_FAMILY_MGGA = 4
XC_FAMILY_LCA = 8
XC_FAMILY_OEP = 16
XC_FAMILY_HYB_GGA = 32
XC_FAMILY_HYB_MGGA = 64
XC_FAMILY_HYB_LDA = 128

XC_FLAGS_HAVE_EXC = (1 << 0)  # = 1
XC_FLAGS_HAVE_VXC = (1 << 1)  # = 2
XC_FLAGS_HAVE_FXC = (1 << 2)  # = 4
XC_FLAGS_HAVE_KXC = (1 << 3)  # = 8
XC_FLAGS_HAVE_LXC = (1 << 4)  # = 16
XC_FLAGS_HAVE_ALL = 31 # The most common case
XC_FLAGS_1D = (1 << 5)  # = 32
XC_FLAGS_2D = (1 << 6)  # = 64
XC_FLAGS_3D = (1 << 7)  # = 128
XC_FLAGS_HYB_CAM = (1 << 8)  # = 256
XC_FLAGS_HYB_CAMY = (1 << 9)  # = 512
XC_FLAGS_VV10 = (1 << 10)  #  1024
XC_FLAGS_HYB_LC = (1 << 11)  #  2048
XC_FLAGS_HYB_LCY = (1 << 12)  #  4096
XC_FLAGS_STABLE = (1 << 13)  #  8192
XC_FLAGS_DEVELOPMENT = (1 << 14)  # 16384
XC_FLAGS_NEEDS_LAPLACIAN = (1 << 15)  # 32768

XC_TAU_EXPLICIT = 0
XC_TAU_EXPANSION = 1

XC_MAX_REFERENCES = 5




"""
Binds the LibXC utility functions.
"""

import ctypes
import numpy as np

from .core import core
from . import flags

### Set required ctypes bindings

core.xc_version.argtypes = (ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
core.xc_version.restype = None

core.xc_version_string.restype = ctypes.c_char_p
core.xc_reference.restype = ctypes.c_char_p
core.xc_reference_doi.restype = ctypes.c_char_p

core.xc_functional_get_number.argtypes = (ctypes.c_char_p, )
core.xc_functional_get_number.restype = ctypes.c_int

core.xc_functional_get_name.argtypes = (ctypes.c_int, )
core.xc_functional_get_name.restype = ctypes.c_char_p

core.xc_family_from_id.argtypes = (ctypes.c_int, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int))
core.xc_family_from_id.restype = ctypes.c_int

core.xc_available_functional_numbers.argtypes = (np.ctypeslib.ndpointer(dtype=np.intc, ndim=1, flags=("W", "C", "A")), )
core.xc_available_functional_numbers_by_name.argtypes = (np.ctypeslib.ndpointer(dtype=np.intc, ndim=1, flags=("W", "C", "A")), )

core.xc_available_functional_names.argtypes = (ctypes.POINTER(ctypes.c_char_p), )

### Build wrapper functions


def xc_version():
    """
    Returns the current LibXC version as semantic versioning tuple.

    Returns
    -------
    version : tuple
        The (major, minor, patch) version of the linked LibXC shared object.

    Examples
    --------
    >>> pylibxc.util.xc_version()
    (4, 0, 1)

    """
    major = ctypes.c_int()
    minor = ctypes.c_int()
    patch = ctypes.c_int()
    core.xc_version(major, minor, patch)
    return (major.value, minor.value, patch.value)


def xc_version_string():
    """
    Returns the current LibXC version as a string.

    Returns
    -------
    version : str
        The string representation of the current LibXC version.

    Examples
    --------
    >>> pylibxc.util.xc_version_string()
    "4.0.1"

    """
    return core.xc_version_string().decode("UTF-8")


def xc_reference():
    """
    Returns the reference for the current LibXC version as a string.

    Returns
    -------
    reference : str
        The string representation of the literature reference for LibXC.

    Examples
    --------
    >>> pylibxc.util.xc_reference()
    "S. Lehtola, C. Steigemann, M. J. Oliveira, and M. A. Marques, SoftwareX 7, 1 (2018)"

    """
    return core.xc_reference().decode("UTF-8")


def xc_reference_doi():
    """
    Returns the doi of the reference for the current LibXC version as a string.

    Returns
    -------
    doi : str
        The string representation of the doi of the literature reference for LibXC.

    Examples
    --------
    >>> pylibxc.util.xc_reference_doi()
    "10.1016/j.softx.2017.11.002"

    """
    return core.xc_reference_doi().decode("UTF-8")


def xc_functional_get_number(name):
    """
    Returns the functional ID from a given string.

    Parameters
    ----------
    name : str
        The functional name to find the ID of.

    Returns
    -------
    id : int
        The ID of the requested functional.

    Examples
    --------
    >>> pylibxc.util.xc_functional_get_number("XC_GGA_X_GAM")
    32
    """
    if not isinstance(name, str):
        raise TypeError("xc_functional_get_number: name must be a string. Got {}".format(name))
    return core.xc_functional_get_number(ctypes.c_char_p(name.encode()))


def xc_functional_get_name(func_id):
    """
    Returns the functional name from a ID.

    Parameters
    ----------
    func_id : int
        The LibXC functional ID

    Returns
    -------
    functional_name : str
        The functional_name of the requested functional.

    Examples
    --------
    >>> pylibxc.util.xc_functional_get_name(32)
    "gga_x_gam"
    """
    if not isinstance(func_id, (int, np.integer)):
        raise TypeError("xc_functional_get_name: func_id must be an int. Got {}".format(func_id))
    ret = core.xc_functional_get_name(func_id)
    if ret is not None:
        ret = ret.decode("UTF-8")
    return ret

def xc_family_from_id(func_id):
    """
    Returns the family class and family number (?).

    Parameters
    ----------
    func_id : int
        The LibXC functional ID

    Returns
    -------
    functional_family : int
        The family ID.
    functional_number : int
        The functional number within a family.

    Examples
    --------
    >>> pylibxc.util.xc_family_from_id(72)
    (4, 4)

    """
    if not isinstance(func_id, (int, np.integer)):
        raise TypeError("xc_family_from_id: func_id must be an int. Got {}".format(func_id))
    family = ctypes.c_int()
    number = ctypes.c_int()
    core.xc_family_from_id(func_id, ctypes.pointer(family), ctypes.pointer(number))

    return (family.value, number.value)


def xc_number_of_functionals():
    """
    Returns the totaly number of XC functionals available in LibXC.

    Returns
    -------
    number_of_functinals : int
        The total number of functionals available in LibXC.

    Examples
    --------
    >>> pylibxc.util.xc_number_of_functionals()
    447
    """

    return core.xc_number_of_functionals()


def xc_available_functional_numbers():
    """
    Returns a list of all available XC functional IDs


    Returns
    -------
    functional_ids : list of ints
        All available LibXC functional IDs.

    Examples
    --------
    >>> xc_func_list = pylibxc.util.xc_available_functional_numbers()
    np.array([1, 2, ..., 568, 569])
    """

    nfunc = xc_number_of_functionals()

    ret = np.zeros(nfunc, dtype=np.intc)
    core.xc_available_functional_numbers(ret)
    return ret


def xc_available_functional_names():
    """
    Returns a list of all available XC functional names

    Returns
    -------
    functional_names : list of strs
        All available LibXC functional names.

    Examples
    --------
    >>> xc_func_list = pylibxc.util.xc_available_functional_names()
    ['lda_x', 'lda_c_wigner', ..., 'hyb_mgga_x_revscan0', 'hyb_mgga_xc_b98']
    """

    # I give up trying to get char** working, someone else can pick it up.
    nfunc = xc_number_of_functionals()
    func_ids = np.zeros(nfunc, dtype=np.intc)
    core.xc_available_functional_numbers_by_name(func_ids)
    available_names = [xc_functional_get_name(x) for x in func_ids]
    return available_names



"""
Tests the LibXCFunctional class.
"""

import pytest
import numpy as np
import ctypes
import pylibxc

compute_test_dim = 5

np.random.seed(0)


def _dict_array_comp(test, ref, keys):
    for key in keys:
        tmp = np.testing.assert_allclose(test[key], ref[key])
        # print(key, np.allclose(test[key], ref[key]), np.linalg.norm(test[key] - ref[key]))
    return True


_size_tuples = {"unpolarized": (1, 1, 1, 1), "polarized": (2, 3, 2, 2)}


def test_libxc_functional_build():

    pylibxc.LibXCFunctional(1, 1)
    pylibxc.LibXCFunctional(1, 2)

    pylibxc.LibXCFunctional("XC_LDA_C_VWN", "polarized")
    pylibxc.LibXCFunctional("lda_c_vwn", "unpolarized")

    # Check functional edge cases
    with pytest.raises(KeyError):
        pylibxc.LibXCFunctional("something", 1)

    with pytest.raises(KeyError):
        pylibxc.LibXCFunctional(5000, 1)

    # Check spin edge cases
    with pytest.raises(KeyError):
        pylibxc.LibXCFunctional("lda_c_vwn", 10)

    with pytest.raises(KeyError):
        pylibxc.LibXCFunctional("lda_c_vwn", "something")


def test_libxc_functional_info():

    func = pylibxc.LibXCFunctional(1, 1)
    assert func.get_number() == 1
    assert func.get_kind() == 0
    assert func.get_name() == "Slater exchange"
    assert func.get_family() == 1
    # XC_FLAGS_3D + [HAVE_EXC | HAVE_VXC | HAVE_FXC | HAVE_KXC]
    assert func.get_flags() in [129, 131, 135, 143, 159]
    assert len(func.get_bibtex()) == 2
    assert len(func.get_references()) == 2
    assert len(func.get_doi()) == 2

    func = pylibxc.LibXCFunctional("XC_HYB_MGGA_XC_WB97M_V", 1)
    assert func.get_number() == 531
    assert func.get_kind() == 2
    assert func.get_name() == "wB97M-V exchange-correlation functional"
    assert func.get_family() == 64
    # XC_FLAGS_3D + XC_FLAGS_HYB_CAM + XC_FLAGS_VV10 + [HAVE_EXC | HAVE_VXC | HAVE_FXC | HAVE_KXC]
    assert func.get_flags() in [1409, 1411, 1415, 1423, 1439]
    assert len(func.get_bibtex()) == 1
    assert len(func.get_references()) == 1
    assert len(func.get_doi()) == 1

def test_ext_params():

    func = pylibxc.LibXCFunctional(1, 1)
    assert 0 == len(func.get_ext_param_descriptions())
    assert 0 == len(func.get_ext_param_default_values())
    func.set_dens_threshold(1.e-16)
    func.set_dens_threshold(5)
    with pytest.raises(ValueError):
        func.set_ext_params([])

    func = pylibxc.LibXCFunctional("XC_HYB_GGA_XC_HSE06", 1)
    assert 3 == len(func.get_ext_param_descriptions())
    assert 3 == len(func.get_ext_param_default_values())
    assert all("param" in x for x in func.get_ext_param_descriptions())
    func.set_dens_threshold(1.e-16)
    func.set_dens_threshold(5)

    # Segfaults, need to check it out
    func.set_ext_params([5, 3, 3])

    with pytest.raises(ValueError):
        func.set_ext_params([5, 3])

    with pytest.raises(ValueError):
        func.set_dens_threshold(-1)


@pytest.mark.parametrize("polar", [("unpolarized"), ("polarized")])
def test_lda_compute(polar):

    # Build input
    ndim = _size_tuples[polar]
    inp = {}
    inp["rho"] = np.random.random((compute_test_dim * ndim[0]))

    func = pylibxc.LibXCFunctional("lda_c_vwn", polar)

    # Check capabilities
    do_l = func._have_lxc
    do_k = func._have_kxc
    do_f = func._have_fxc
    do_v = func._have_vxc
    do_e = func._have_exc

    # Compute
    ret_full = func.compute(inp, do_exc=do_e, do_vxc=do_v, do_fxc=do_f, do_kxc=do_k, do_lxc=do_l)
    ret_ev = func.compute(inp, do_exc=do_e, do_vxc=do_v, do_fxc=False, do_kxc=False, do_lxc=False)
    ret_e = func.compute(inp, do_exc=do_e, do_vxc=False, do_fxc=False, do_kxc=False, do_lxc=False)
    ret_v = func.compute(inp, do_exc=False, do_vxc=do_v, do_fxc=False, do_kxc=False, do_lxc=False)
    ret_f = func.compute(inp, do_exc=False, do_vxc=False, do_fxc=do_f, do_kxc=False, do_lxc=False)
    ret_k = func.compute(inp, do_exc=False, do_vxc=False, do_fxc=False, do_kxc=do_k, do_lxc=False)
    ret_l = func.compute(inp, do_exc=False, do_vxc=False, do_fxc=False, do_kxc=False, do_lxc=do_l)

    # Test consistency
    if do_e:
        assert ret_full["zk"].size == compute_test_dim
    if do_v:
        assert ret_full["vrho"].size == compute_test_dim * ndim[0]

    if do_e:
        assert np.allclose(ret_full["zk"], ret_ev["zk"])
    if do_v:
        assert np.allclose(ret_full["vrho"], ret_ev["vrho"])

    if do_e:
        assert np.allclose(ret_full["zk"], ret_e["zk"])
    if do_v:
        assert np.allclose(ret_full["vrho"], ret_v["vrho"])
    if do_f:
        assert np.allclose(ret_full["v2rho2"], ret_f["v2rho2"])
    if do_k:
        assert np.allclose(ret_full["v3rho3"], ret_k["v3rho3"])
    if do_l:
        assert np.allclose(ret_full["v4rho4"], ret_l["v4rho4"])


@pytest.mark.parametrize("polar", [("unpolarized"), ("polarized")])
def test_gga_compute(polar):

    # Build input
    ndim = _size_tuples[polar]
    inp = {}
    inp["rho"] = np.random.random((compute_test_dim * ndim[0]))
    inp["sigma"] = np.random.random((compute_test_dim * ndim[1]))

    # Compute
    func = pylibxc.LibXCFunctional("gga_c_pbe", polar)

    # Check capabilities
    do_l = func._have_lxc
    do_k = func._have_kxc
    do_f = func._have_fxc
    do_v = func._have_vxc
    do_e = func._have_exc

    # Compute
    ret_full = func.compute(inp, do_exc=do_e, do_vxc=do_v, do_fxc=do_f, do_kxc=do_k, do_lxc=do_l)
    ret_ev = func.compute(inp, do_exc=do_e, do_vxc=do_v, do_fxc=False, do_kxc=False, do_lxc=False)
    ret_e = func.compute(inp, do_exc=do_e, do_vxc=False, do_fxc=False, do_kxc=False, do_lxc=False)
    ret_v = func.compute(inp, do_exc=False, do_vxc=do_v, do_fxc=False, do_kxc=False, do_lxc=False)
    ret_f = func.compute(inp, do_exc=False, do_vxc=False, do_fxc=do_f, do_kxc=False, do_lxc=False)
    ret_k = func.compute(inp, do_exc=False, do_vxc=False, do_fxc=False, do_kxc=do_k, do_lxc=False)
    ret_l = func.compute(inp, do_exc=False, do_vxc=False, do_fxc=False, do_kxc=False, do_lxc=do_l)

    # Test consistency
    if do_e:
        assert ret_full["zk"].size == compute_test_dim
    if do_v:
        assert ret_full["vrho"].size == compute_test_dim * ndim[0]
        assert ret_full["vsigma"].size == compute_test_dim * ndim[1]

    if do_e and do_v:
        assert _dict_array_comp(ret_full, ret_ev, ["zk", "vrho", "vsigma"])

    if do_e:
        assert _dict_array_comp(ret_full, ret_e, ["zk"])
    if do_v:
        assert _dict_array_comp(ret_full, ret_v, ["vrho", "vsigma"])
    if do_f:
        assert _dict_array_comp(ret_full, ret_f, ["v2rho2", "v2rhosigma", "v2sigma2"])
    if do_k:
        assert _dict_array_comp(ret_full, ret_k, ["v3rho3", "v3rho2sigma", "v3rhosigma2", "v3sigma3"])
    if do_l:
        assert _dict_array_comp(ret_full, ret_l, ["v4rho4", "v4rho3sigma", "v4rho2sigma2", "v4rhosigma3", "v4sigma4"])


@pytest.mark.parametrize("polar", [("unpolarized"), ("polarized")])
def test_mgga_compute(polar):

    # Build input
    ndim = _size_tuples[polar]

    inp = {}
    inp["rho"] = np.random.random((compute_test_dim * ndim[0]))
    inp["sigma"] = np.random.random((compute_test_dim * ndim[1]))
    inp["tau"] = np.random.random((compute_test_dim * ndim[3]))

    # Compute
    func = pylibxc.LibXCFunctional("mgga_c_tpss", polar)

    # Test consistency
    ret_ev = func.compute(inp, do_exc=True, do_vxc=True)
    ret_e = func.compute(inp, do_exc=True, do_vxc=False)
    ret_v = func.compute(inp, do_exc=False, do_vxc=True)

    assert ret_ev["zk"].size == compute_test_dim
    assert ret_ev["vrho"].size == compute_test_dim * ndim[0]
    assert ret_ev["vsigma"].size == compute_test_dim * ndim[1]

    assert _dict_array_comp(ret_ev, ret_e, ["zk"])
    assert _dict_array_comp(ret_ev, ret_v, ["vrho", "vsigma", "vtau"])


@pytest.mark.parametrize("polar", [("unpolarized"), ("polarized")])
def test_mgga_lapl_compute(polar):

    # Build input
    ndim = _size_tuples[polar]

    inp = {}
    inp["rho"] = np.random.random((compute_test_dim * ndim[0]))
    inp["sigma"] = np.random.random((compute_test_dim * ndim[1]))
    inp["tau"] = np.random.random((compute_test_dim * ndim[3]))
    inp["lapl"] = np.random.random((compute_test_dim * ndim[3]))

    # Compute
    func = pylibxc.LibXCFunctional("mgga_x_br89", polar)

    # Test consistency
    ret_ev = func.compute(inp, do_exc=True, do_vxc=True)
    ret_e = func.compute(inp, do_exc=True, do_vxc=False)
    ret_v = func.compute(inp, do_exc=False, do_vxc=True)

    assert ret_ev["zk"].size == compute_test_dim
    assert ret_ev["vrho"].size == compute_test_dim * ndim[0]
    assert ret_ev["vsigma"].size == compute_test_dim * ndim[1]

    assert _dict_array_comp(ret_ev, ret_e, ["zk"])
    assert _dict_array_comp(ret_ev, ret_v, ["vrho", "vsigma", "vtau"])

    # Test exception
    del inp["lapl"]
    with pytest.raises(KeyError):
        func.compute(inp, do_exc=True, do_vxc=True)


@pytest.mark.parametrize("polar", [("unpolarized"), ("polarized")])
def test_deriv_flags(polar):
    func = pylibxc.LibXCFunctional("mgga_c_tpss", polar)

    ndim = _size_tuples[polar]
    inp = {}
    inp["rho"] = np.random.random((compute_test_dim * ndim[0]))
    inp["sigma"] = np.random.random((compute_test_dim * ndim[1]))
    inp["tau"] = np.random.random((compute_test_dim * ndim[3]))
    inp["lapl"] = np.random.random((compute_test_dim * ndim[3]))

    # disabled as mgga_c_tpss now has fxc
    # please fix this better!
    #with pytest.raises(ValueError):
    #    func.compute(inp, do_fxc=True)

def test_hyb_getters():

    func = pylibxc.LibXCFunctional("hyb_gga_xc_b3lyp", "unpolarized")
    assert pytest.approx(0.2) == func.get_hyb_exx_coef()

    with pytest.raises(ValueError):
        func.get_cam_coef()
    with pytest.raises(ValueError):
        func.get_vv10_coef()


def test_cam_getters():

    func = pylibxc.LibXCFunctional("hyb_gga_xc_cam_b3lyp", "unpolarized")

    with pytest.raises(ValueError):
        assert pytest.approx(0.65) == func.get_hyb_exx_coef()

    omega, alpha, beta = func.get_cam_coef()
    assert pytest.approx(0.33) == omega
    assert pytest.approx(0.65) == alpha
    assert pytest.approx(-0.46) == beta

    with pytest.raises(ValueError):
        func.get_vv10_coef()


def test_vv10_getters():

    func = pylibxc.LibXCFunctional("gga_xc_vv10", "unpolarized")

    b, C = func.get_vv10_coef()
    assert pytest.approx(5.9) == b
    assert pytest.approx(0.0093) == C

    with pytest.raises(ValueError):
        func.get_cam_coef()

    with pytest.raises(ValueError):
        func.get_hyb_exx_coef()



import os
import re
import sys
import platform
import subprocess

from setuptools import setup, find_packages, Extension
from setuptools.command.build_ext import build_ext
from distutils.version import LooseVersion


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the following extensions: " +
                               ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)', out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir]
        cmake_args += ['-DBUILD_SHARED_LIBS=ON']
        cmake_args += ['-DBUILD_TESTING=OFF']
        # Python tests need third derivatives
        cmake_args += ['-DDISABLE_KXC=OFF']

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(cfg.upper(), extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            nprocs = 1
            try:
                import multiprocessing as mp
                nprocs = mp.cpu_count()
            except ImportError:
                pass
            build_args += ['--', '-j{}'.format(nprocs)]

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''), self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args, cwd=self.build_temp)


if __name__ == "__main__":
    exec(open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "pylibxc/version.py")).read())
    setup(
        name='pylibxc',
        version=__version__,
        description=
        'PyLibxc is a python-bound C library of exchange and correlation functionals for density-functional (DFT) theory.',
        author='LibXC Authors',
        author_email='marques@tddft.org',
        url="https://gitlab.com/libxc/libxc",
        license='MPL',
        install_requires=[
            'numpy>=1.7',
        ],
        extras_require={
            'docs': [
                'sphinx==1.2.3',    # autodoc was broken in 1.3.1
                'sphinxcontrib-napoleon',
                'sphinx_rtd_theme',
                'numpydoc',
            ],
            'tests': [
                'pytest',
                'pytest-cov',
            ],
        },
        tests_require=[
            'pytest',
            'pytest-cov',
        ],
        packages=find_packages(),
        ext_modules=[CMakeExtension('pylibxc.libxc')],
        cmdclass=dict(build_ext=CMakeBuild),
        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Science/Research',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
        ],
        zip_safe=False,
        long_description="""
Libxc is a library of exchange-correlation functionals for
density-functional theory. The aim is to provide a portable, well
tested and reliable set of exchange and correlation functionals that
can be used by a variety of programs.

For more information, please check the manual at
http://www.tddft.org/programs/Libxc
""")
